#include "OGLFT.h"

#include <GL/gl.h>
#include <GL/glu.h>

#include "DrawingsGL.h"

#include "OverlayBase.h"
#include "UserInterface.h"
#include "Model.h"
#include "View.h"
#include "Sprite.h"
#include "ShipEntity.h"

#include "Engine.h"
#include "Game.h"

namespace asteroids {

// static float left = 0;
static float right = 0;
// static float top = 0;
static float bottom = 0;

void MainMenuOverlay::Init () {
	// setup of the HUD font
	GetView()->LoadFont ("AldotheApache.ttf", 20);
	GetView()->SelectFont ("AldotheApache.ttf");
	GetView()->SetFontJustification (Engine::FontJustificationRight);
}

bool MainMenuOverlay::OnKeyDown (const SDL_keysym &keysym) {
	switch (keysym.sym) {
		case SDLK_ESCAPE:
			Engine::RunCommand ("quit");
			return true;
		case SDLK_RETURN:
			GetModel()->SetGameState(GameStateRunning);
			return true;
		default:
			return true;
	}
}

void MainMenuOverlay::Draw () {
	glClearColor (0.1, 0.1, 0.1, 1.);

	right = static_cast<float> (Engine::GetWindowWidth());
	bottom = static_cast<float> (Engine::GetWindowHeight());

	// we switch to orthographic projection and draw the contents of the 2d 
	// overlay on top of the previous drawings
	glMatrixMode (GL_PROJECTION);
	glPushMatrix ();
	glLoadIdentity ();

	// first we have to get the size of the current viewport to set up the
	// orthographic projection correctly
	GLint viewport[4];
	glGetIntegerv(GL_VIEWPORT, viewport);
	gluOrtho2D (viewport[0], viewport[2], viewport[3], viewport[1]);

	glMatrixMode (GL_MODELVIEW);
	glPushMatrix ();
	glLoadIdentity ();

	GetView()->SelectFont("console.ttf");

	// then we do the drawings
	Engine::DrawGLString ( right * 0.5 - 100, bottom * 0.5 - 8 - 64, "A s t e r o i d s");
	Engine::DrawGLString ( right * 0.5 - 100, bottom * 0.5 - 8 - 32, "Main Menu");
	Engine::DrawGLString ( right * 0.5 - 80, bottom * 0.5 - 8 - 16, "[Return] - Start Game");
	Engine::DrawGLString ( right * 0.5 - 80, bottom * 0.5 - 8, "[Escape] - Quit");

	glPopMatrix ();

	glMatrixMode (GL_PROJECTION);
	glPopMatrix ();

	glMatrixMode (GL_MODELVIEW);

};

/**********************
 *
 * Game Running
 *
 **********************/

void GameRunningOverlay::Init () {
	if (!mShipSprite.LoadFromPNG("./data/textures/ship.png"))
		Engine::LogError ("Could not load ship sprite!");

	// setup of the HUD font
	GetView()->LoadFont ("AldotheApache.ttf", 20);
	GetView()->SelectFont ("AldotheApache.ttf");
	GetView()->SetFontJustification (Engine::FontJustificationRight);

	assert (mShipSprite.GetWidth() > 1);
	mShipSprite.SetScale (0.1);
}

bool GameRunningOverlay::OnKeyDown (const SDL_keysym &keysym) {
	switch (keysym.sym) {
		case SDLK_ESCAPE:
			GetModel()->SetGameState(GameStatePaused);
			return true;
		default:
			break;
	}

	return false;
}

void GameRunningOverlay::Draw () {
	glClearColor (0., 0., 0., 1.);

	right = static_cast<float> (Engine::GetWindowWidth());
	bottom = static_cast<float> (Engine::GetWindowHeight());

	// we switch to orthographic projection and draw the contents of the 2d 
	// overlay on top of the previous drawings
	glMatrixMode (GL_PROJECTION);
	glPushMatrix ();
	glLoadIdentity ();

	// first we have to get the size of the current viewport to set up the
	// orthographic projection correctly
	GLint viewport[4];
	glGetIntegerv(GL_VIEWPORT, viewport);
	gluOrtho2D (viewport[0], viewport[2], viewport[3], viewport[1]);

	glMatrixMode (GL_MODELVIEW);
	glPushMatrix ();
	glLoadIdentity ();

	// then we do the drawings
	right = static_cast<float> (Engine::GetWindowWidth());
	bottom = static_cast<float> (Engine::GetWindowHeight());

	GetView()->SelectFont ("AldotheApache.ttf");

	std::ostringstream out_stream;
	out_stream << GetModel()->GetPlayerLives() << " x ";
	GetView()->DrawGLString (right - 64, bottom - 20, out_stream.str().c_str()); 

	mShipSprite.DrawAt2D (right - 32 - 10, bottom - 16);

	out_stream.str("");
	out_stream << GetModel()->GetPoints();
	GetView()->DrawGLString (right - 10, 40, out_stream.str().c_str()); 

	glPopMatrix ();

	glMatrixMode (GL_PROJECTION);
	glPopMatrix ();

	glMatrixMode (GL_MODELVIEW);

};

/**********************
 *
 * Game Over
 *
 **********************/

void GameOverOverlay::Init () {
}

bool GameOverOverlay::OnKeyDown (const SDL_keysym &keysym) {
	switch (keysym.sym) {
		case SDLK_ESCAPE:
			GetModel()->SetGameState(GameStateMainMenu);
			break;
		case SDLK_RETURN:
			GetModel()->SetGameState(GameStateMainMenu);
			break;
		default:
			break;
	}

	return true;
}

void GameOverOverlay::Draw () {
	glClearColor (0., 0., 0., 1.);

	right = static_cast<float> (Engine::GetWindowWidth());
	bottom = static_cast<float> (Engine::GetWindowHeight());

	// we switch to orthographic projection and draw the contents of the 2d 
	// overlay on top of the previous drawings
	glMatrixMode (GL_PROJECTION);
	glPushMatrix ();
	glLoadIdentity ();

	// first we have to get the size of the current viewport to set up the
	// orthographic projection correctly
	GLint viewport[4];
	glGetIntegerv(GL_VIEWPORT, viewport);
	gluOrtho2D (viewport[0], viewport[2], viewport[3], viewport[1]);

	glMatrixMode (GL_MODELVIEW);
	glPushMatrix ();
	glLoadIdentity ();

	GetView()->SelectFont("console.ttf");

	// then we do the drawings
	std::ostringstream topbar_stream;
	
	Engine::DrawGLString ( right * 0.5 - 60, bottom * 0.5 - 8 - 64, "G a m e  O v e r");
	
	if (GetModel()->GetPlayerLives() == 0) {
		topbar_stream << "That was pathetic! ";
		Engine::DrawGLString ( right * 0.5 - 80, bottom * 0.5 - 8, topbar_stream.str().c_str ());
	}	else {
		Engine::DrawGLString ( right * 0.5 - 80, bottom * 0.5 - 32, "You earned yourself the");
		Engine::DrawGLString ( right * 0.5 - 60, bottom * 0.5 + 0, "You Rock(TM) award.");
		Engine::DrawGLString ( right * 0.5 - 70, bottom * 0.5 + 32, "Go tell your friends.");
	}

	glPopMatrix ();

	glMatrixMode (GL_PROJECTION);
	glPopMatrix ();

	glMatrixMode (GL_MODELVIEW);

};

/**********************
 *
 * Level Complete
 *
 **********************/

void LevelCompleteOverlay::Init () {
}

bool LevelCompleteOverlay::OnKeyDown (const SDL_keysym &keysym) {
	switch (keysym.sym) {
		case SDLK_RETURN:
			GetModel()->SetGameState(GameStateRunning);
			break;
		default:
			break;
	}
	return true;
}

void LevelCompleteOverlay::Draw () {
	glClearColor (0., 0., 0., 1.);

	right = static_cast<float> (Engine::GetWindowWidth());
	bottom = static_cast<float> (Engine::GetWindowHeight());

	// we switch to orthographic projection and draw the contents of the 2d 
	// overlay on top of the previous drawings
	glMatrixMode (GL_PROJECTION);
	glPushMatrix ();
	glLoadIdentity ();

	// first we have to get the size of the current viewport to set up the
	// orthographic projection correctly
	GLint viewport[4];
	glGetIntegerv(GL_VIEWPORT, viewport);
	gluOrtho2D (viewport[0], viewport[2], viewport[3], viewport[1]);

	glMatrixMode (GL_MODELVIEW);
	glPushMatrix ();
	glLoadIdentity ();

	GetView()->SelectFont("console.ttf");

	// then we do the drawings
	Engine::DrawGLString ( right * 0.5 - 80, bottom * 0.5 - 8 - 16, "Congratulations - You rock!");
	Engine::DrawGLString ( right * 0.5 - 80, bottom * 0.5 - 8, "[Return] - Next level ...");

	glPopMatrix ();

	glMatrixMode (GL_PROJECTION);
	glPopMatrix ();

	glMatrixMode (GL_MODELVIEW);

};

/**********************
 *
 * Game Paused
 *
 **********************/

void GamePausedOverlay::Init () {
}

bool GamePausedOverlay::OnKeyDown (const SDL_keysym &keysym) {
	switch (keysym.sym) {
		case SDLK_ESCAPE:
			Engine::RunCommand ("quit");
			return true;
		case SDLK_RETURN:
			GetModel()->SetGameState(GameStateRunning);
			return true;
		default:
			return true;
	}
}

void GamePausedOverlay::Draw () {
	glClearColor (0.1, 0.1, 0.1, 1.);

	right = static_cast<float> (Engine::GetWindowWidth());
	bottom = static_cast<float> (Engine::GetWindowHeight());

	// we switch to orthographic projection and draw the contents of the 2d 
	// overlay on top of the previous drawings
	glMatrixMode (GL_PROJECTION);
	glPushMatrix ();
	glLoadIdentity ();

	// first we have to get the size of the current viewport to set up the
	// orthographic projection correctly
	GLint viewport[4];
	glGetIntegerv(GL_VIEWPORT, viewport);
	gluOrtho2D (viewport[0], viewport[2], viewport[3], viewport[1]);

	glMatrixMode (GL_MODELVIEW);
	glPushMatrix ();
	glLoadIdentity ();

	GetView()->SelectFont("console.ttf");

	// then we do the drawings
	Engine::DrawGLString ( right * 0.5 - 100, bottom * 0.5 - 8 - 64, "A s t e r o i d s");
	Engine::DrawGLString ( right * 0.5 - 100, bottom * 0.5 - 8 - 32, "Main Menu");
	Engine::DrawGLString ( right * 0.5 - 80, bottom * 0.5 - 8 - 16, "[Return] - Resume Game");
	Engine::DrawGLString ( right * 0.5 - 80, bottom * 0.5 - 8, "[Escape] - Quit");

	glPopMatrix ();

	glMatrixMode (GL_PROJECTION);
	glPopMatrix ();

	glMatrixMode (GL_MODELVIEW);

};

/**********************
 *
 * Player Died
 *
 **********************/

void PlayerDiedOverlay::Init () {
}

bool PlayerDiedOverlay::OnKeyDown (const SDL_keysym &keysym) {
	switch (keysym.sym) {
		case SDLK_RETURN:
			GetModel()->SetGameState(GameStateRunning);
			break;
		default:
			break;
	}
	return true;
}

void PlayerDiedOverlay::Draw () {
	glClearColor (0., 0., 0., 1.);

	right = static_cast<float> (Engine::GetWindowWidth());
	bottom = static_cast<float> (Engine::GetWindowHeight());

	// we switch to orthographic projection and draw the contents of the 2d 
	// overlay on top of the previous drawings
	glMatrixMode (GL_PROJECTION);
	glPushMatrix ();
	glLoadIdentity ();

	// first we have to get the size of the current viewport to set up the
	// orthographic projection correctly
	GLint viewport[4];
	glGetIntegerv(GL_VIEWPORT, viewport);
	gluOrtho2D (viewport[0], viewport[2], viewport[3], viewport[1]);

	glMatrixMode (GL_MODELVIEW);
	glPushMatrix ();
	glLoadIdentity ();

	GetView()->SelectFont("console.ttf");

	// then we do the drawings
	std::ostringstream topbar_stream;
	topbar_stream << "You died ...";
	Engine::DrawGLString ( right * 0.5 - 40, bottom * 0.5 - 8 - 32, topbar_stream.str().c_str ());

	Engine::DrawGLString ( right * 0.5 - 90, bottom * 0.5 + 8, "Press [Return] to continue.");

	glPopMatrix ();

	glMatrixMode (GL_PROJECTION);
	glPopMatrix ();

	glMatrixMode (GL_MODELVIEW);

};

}
